from tokenize import String
import bpy
from bpy.types import PropertyGroup
from bpy.props import (
    BoolProperty,
    FloatProperty,
    FloatVectorProperty,
    EnumProperty,
    IntProperty,
    StringProperty,
    PointerProperty,
    CollectionProperty,
)

from . handlers import *


class PA_GeneralSettings(PropertyGroup):
    ############################################################################
    # General Physical Celestial Objects Properties
    ############################################################################

    enabled: BoolProperty(
        name="Enable celestial objects",
        default=False,
        update=enable_celestial_objects,
    )

    use_standalone_lighting: BoolProperty(
        name="Use addon's standalone lighting rather than PSA",
        default=True,
        update=set_standalone_lighting,
    )

    sun: PointerProperty(
        name="Sun",
        type=bpy.types.Object,
        update=update_sun,
    )


class PA_NodeReference(PropertyGroup):
    ############################################################################
    # A generic PropertyGroup used to store node information
    ############################################################################

    node_name: StringProperty(
        name="Node name",
        default="",
        description="Name of referenced node",
    )


class PA_Exposed(PropertyGroup):
    ############################################################################
    # Exposed pointers and parameters
    ############################################################################

    inputs_node: PointerProperty(
        name="Physical Inputs",
        type=bpy.types.NodeTree,
    )

    celestial_node: PointerProperty(
        name="Physical Celestial Objects",
        type=bpy.types.NodeTree,
    )

    # Nodes not directly linked to a specific instance
    depth_nodes: CollectionProperty(
        name="Depth sorting node names",
        type=PA_NodeReference,
    )

    shadow_nodes: CollectionProperty(
        name="Shadow node names",
        type=PA_NodeReference,
    )

    indirect_nodes: CollectionProperty(
        name="Indirect node names",
        type=PA_NodeReference,
    )

    transform_nodes: CollectionProperty(
        name="Planet transform node names",
        type=PA_NodeReference,
    )

    planet_data_nodes: CollectionProperty(
        name="Planet data node names",
        type=PA_NodeReference,
    )

    material_group_names: CollectionProperty(
        name="Material data group names",
        type=PA_NodeReference,
    )

    height_normal_group_names: CollectionProperty(
        name="Height to normal group names",
        type=PA_NodeReference,
    )

    atmosphere_instance: IntProperty(
        name="Atmosphere enabled instance index",
        default = -1,
    )

    ring_instance: IntProperty(
        name="Rings enabled instance index",
        default = -1,
    )

    celestial_node_name: StringProperty(
        name="Celestial node name",
    )

